@@ -1,8 +1,6 @@
 """
 Displays Agg images in the browser, with interactivity
 """
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
 
 # The WebAgg backend is divided into two modules:
 #
@@ -13,17 +11,17 @@ from __future__ import (absolute_import, division, print_function,
 # - `backend_webagg.py` contains a concrete implementation of a basic
 #   application, implemented with tornado.
 
-import six
-
+from contextlib import contextmanager
 import errno
+from io import BytesIO
 import json
 import os
+from pathlib import Path
 import random
 import sys
 import signal
 import socket
 import threading
-from contextlib import contextmanager
 
 try:
     import tornado
@@ -34,11 +32,8 @@ import tornado.web
 import tornado.ioloop
 import tornado.websocket
 
-import matplotlib
 from matplotlib import rcParams
-from matplotlib import backend_bases
 from matplotlib.backend_bases import _Backend
-from matplotlib.figure import Figure
 from matplotlib._pylab_helpers import Gcf
 from . import backend_webagg_core as core
 from .backend_webagg_core import TimerTornado
@@ -48,12 +43,14 @@ class ServerThread(threading.Thread):
     def run(self):
         tornado.ioloop.IOLoop.instance().start()
 
+
 webagg_server_thread = ServerThread()
 
 
 class FigureCanvasWebAgg(core.FigureCanvasWebAggCore):
     def show(self):
         # show the figure window
+        global show  # placates pyflakes: created by @_Backend.export below
         show()
 
     def new_timer(self, *args, **kwargs):
@@ -66,20 +63,14 @@ class WebAggApplication(tornado.web.Application):
 
     class FavIcon(tornado.web.RequestHandler):
         def get(self):
-            image_path = os.path.join(
-                os.path.dirname(os.path.dirname(__file__)),
-                'mpl-data', 'images')
-
             self.set_header('Content-Type', 'image/png')
-            with open(os.path.join(image_path,
-                                   'matplotlib.png'), 'rb') as fd:
-                self.write(fd.read())
+            image_path = Path(rcParams["datapath"], "images", "matplotlib.png")
+            self.write(image_path.read_bytes())
 
     class SingleFigurePage(tornado.web.RequestHandler):
-        def __init__(self, application, request, **kwargs):
-            self.url_prefix = kwargs.pop('url_prefix', '')
-            return tornado.web.RequestHandler.__init__(self, application,
-                                                       request, **kwargs)
+        def __init__(self, application, request, *, url_prefix='', **kwargs):
+            self.url_prefix = url_prefix
+            super().__init__(application, request, **kwargs)
 
         def get(self, fignum):
             fignum = int(fignum)
@@ -96,10 +87,9 @@ class WebAggApplication(tornado.web.Application):
                 canvas=manager.canvas)
 
     class AllFiguresPage(tornado.web.RequestHandler):
-        def __init__(self, application, request, **kwargs):
-            self.url_prefix = kwargs.pop('url_prefix', '')
-            return tornado.web.RequestHandler.__init__(self, application,
-                                                       request, **kwargs)
+        def __init__(self, application, request, *, url_prefix='', **kwargs):
+            self.url_prefix = url_prefix
+            super().__init__(application, request, **kwargs)
 
         def get(self):
             ws_uri = 'ws://{req.host}{prefix}/'.format(req=self.request,
@@ -138,7 +128,7 @@ class WebAggApplication(tornado.web.Application):
 
             self.set_header('Content-Type', mimetypes.get(fmt, 'binary'))
 
-            buff = six.BytesIO()
+            buff = BytesIO()
             manager.canvas.figure.savefig(buff, format=fmt)
             self.write(buff.getvalue())
 
@@ -186,7 +176,7 @@ class WebAggApplication(tornado.web.Application):
             assert url_prefix[0] == '/' and url_prefix[-1] != '/', \
                 'url_prefix must start with a "/" and not end with one.'
 
-        super(WebAggApplication, self).__init__(
+        super().__init__(
             [
                 # Static files for the CSS and JS
                 (url_prefix + r'/_static/(.*)',
@@ -217,7 +207,7 @@ class WebAggApplication(tornado.web.Application):
             template_path=core.FigureManagerWebAgg.get_static_file_path())
 
     @classmethod
-    def initialize(cls, url_prefix='', port=None):
+    def initialize(cls, url_prefix='', port=None, address=None):
         if cls.initialized:
             return
 
@@ -240,20 +230,21 @@ class WebAggApplication(tornado.web.Application):
             for i in range(n - 5):
                 yield port + random.randint(-2 * n, 2 * n)
 
-        success = None
+        if address is None:
+            cls.address = rcParams['webagg.address']
+        else:
+            cls.address = address
         cls.port = rcParams['webagg.port']
         for port in random_ports(cls.port, rcParams['webagg.port_retries']):
             try:
-                app.listen(port)
+                app.listen(port, cls.address)
             except socket.error as e:
                 if e.errno != errno.EADDRINUSE:
                     raise
             else:
                 cls.port = port
-                success = True
                 break
-
-        if not success:
+        else:
             raise SystemExit(
                 "The webagg server could not be started because an available "
                 "port could not be found")
@@ -306,13 +297,9 @@ def ipython_inline_display(figure):
     if not webagg_server_thread.is_alive():
         webagg_server_thread.start()
 
-    with open(os.path.join(
-            core.FigureManagerWebAgg.get_static_file_path(),
-            'ipython_inline_figure.html')) as fd:
-        tpl = fd.read()
-
     fignum = figure.number
-
+    tpl = Path(core.FigureManagerWebAgg.get_static_file_path(),
+               "ipython_inline_figure.html").read_text()
     t = tornado.template.Template(tpl)
     return t.generate(
         prefix=WebAggApplication.url_prefix,
