@@ -1,29 +1,19 @@
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
-import six
-
-import math
-
 import numpy as np
-import numpy.ma as ma
 
 import matplotlib
-rcParams = matplotlib.rcParams
+from matplotlib import rcParams
 from matplotlib.axes import Axes
-from matplotlib import cbook
+import matplotlib.axis as maxis
 from matplotlib.patches import Circle
 from matplotlib.path import Path
 import matplotlib.spines as mspines
-import matplotlib.axis as maxis
-from matplotlib.ticker import Formatter, Locator, NullLocator, FixedLocator, NullFormatter
-from matplotlib.transforms import Affine2D, Affine2DBase, Bbox, \
-    BboxTransformTo, IdentityTransform, Transform, TransformWrapper
+from matplotlib.ticker import (
+    Formatter, NullLocator, FixedLocator, NullFormatter)
+from matplotlib.transforms import Affine2D, BboxTransformTo, Transform
+
 
 class GeoAxes(Axes):
-    """
-    An abstract base class for geographic projections
-    """
+    """An abstract base class for geographic projections."""
     class ThetaFormatter(Formatter):
         """
         Used to format the theta tick labels.  Converts the native
@@ -38,7 +28,7 @@ class GeoAxes(Axes):
             if rcParams['text.usetex'] and not rcParams['text.latex.unicode']:
                 return r"$%0.0f^\circ$" % degrees
             else:
-                return "%0.0f\u00b0" % degrees
+                return "%0.0f\N{DEGREE SIGN}" % degrees
 
     RESOLUTION = 75
 
@@ -87,23 +77,23 @@ class GeoAxes(Axes):
         # This is the transform for longitude ticks.
         self._xaxis_pretransform = \
             Affine2D() \
-            .scale(1.0, self._longitude_cap * 2.0) \
-            .translate(0.0, -self._longitude_cap)
+            .scale(1, self._longitude_cap * 2) \
+            .translate(0, -self._longitude_cap)
         self._xaxis_transform = \
             self._xaxis_pretransform + \
             self.transData
         self._xaxis_text1_transform = \
-            Affine2D().scale(1.0, 0.0) + \
+            Affine2D().scale(1, 0) + \
             self.transData + \
-            Affine2D().translate(0.0, 4.0)
+            Affine2D().translate(0, 4)
         self._xaxis_text2_transform = \
-            Affine2D().scale(1.0, 0.0) + \
+            Affine2D().scale(1, 0) + \
             self.transData + \
-            Affine2D().translate(0.0, -4.0)
+            Affine2D().translate(0, -4)
 
         # This is the transform for latitude ticks.
-        yaxis_stretch = Affine2D().scale(np.pi * 2.0, 1.0).translate(-np.pi, 0.0)
-        yaxis_space = Affine2D().scale(1.0, 1.1)
+        yaxis_stretch = Affine2D().scale(np.pi * 2, 1).translate(-np.pi, 0)
+        yaxis_space = Affine2D().scale(1, 1.1)
         self._yaxis_transform = \
             yaxis_stretch + \
             self.transData
@@ -115,23 +105,23 @@ class GeoAxes(Axes):
              self.transAxes)
         self._yaxis_text1_transform = \
             yaxis_text_base + \
-            Affine2D().translate(-8.0, 0.0)
+            Affine2D().translate(-8, 0)
         self._yaxis_text2_transform = \
             yaxis_text_base + \
-            Affine2D().translate(8.0, 0.0)
+            Affine2D().translate(8, 0)
 
     def _get_affine_transform(self):
         transform = self._get_core_transform(1)
         xscale, _ = transform.transform_point((np.pi, 0))
-        _, yscale = transform.transform_point((0, np.pi / 2.0))
+        _, yscale = transform.transform_point((0, np.pi / 2))
         return Affine2D() \
             .scale(0.5 / xscale, 0.5 / yscale) \
             .translate(0.5, 0.5)
 
     def get_xaxis_transform(self,which='grid'):
-        if which not in ['tick1','tick2','grid']:
-            msg = "'which' must be on of [ 'tick1' | 'tick2' | 'grid' ]"
-            raise ValueError(msg)
+        if which not in ['tick1', 'tick2', 'grid']:
+            raise ValueError(
+                "'which' must be one of 'tick1', 'tick2', or 'grid'")
         return self._xaxis_transform
 
     def get_xaxis_text1_transform(self, pad):
@@ -141,9 +131,9 @@ class GeoAxes(Axes):
         return self._xaxis_text2_transform, 'top', 'center'
 
     def get_yaxis_transform(self,which='grid'):
-        if which not in ['tick1','tick2','grid']:
-            msg = "'which' must be one of [ 'tick1' | 'tick2' | 'grid' ]"
-            raise ValueError(msg)
+        if which not in ['tick1', 'tick2', 'grid']:
+            raise ValueError(
+                "'which' must be one of 'tick1', 'tick2', or 'grid'")
         return self._yaxis_transform
 
     def get_yaxis_text1_transform(self, pad):
@@ -174,8 +164,7 @@ class GeoAxes(Axes):
 
     def format_coord(self, lon, lat):
         'return a format string formatting the coordinate'
-        lon = lon * (180.0 / np.pi)
-        lat = lat * (180.0 / np.pi)
+        lon, lat = np.rad2deg([lon, lat])
         if lat >= 0.0:
             ns = 'N'
         else:
@@ -184,35 +173,32 @@ class GeoAxes(Axes):
             ew = 'E'
         else:
             ew = 'W'
-        return '%f\u00b0%s, %f\u00b0%s' % (abs(lat), ns, abs(lon), ew)
+        return ('%f\N{DEGREE SIGN}%s, %f\N{DEGREE SIGN}%s'
+                % (abs(lat), ns, abs(lon), ew))
 
     def set_longitude_grid(self, degrees):
         """
         Set the number of degrees between each longitude grid.
         """
-        number = (360.0 / degrees) + 1
-        self.xaxis.set_major_locator(
-            FixedLocator(
-                np.linspace(-np.pi, np.pi, number, True)[1:-1]))
-        self._logitude_degrees = degrees
+        # Skip -180 and 180, which are the fixed limits.
+        grid = np.arange(-180 + degrees, 180, degrees)
+        self.xaxis.set_major_locator(FixedLocator(np.deg2rad(grid)))
         self.xaxis.set_major_formatter(self.ThetaFormatter(degrees))
 
     def set_latitude_grid(self, degrees):
         """
-        Set the number of degrees between each longitude grid.
+        Set the number of degrees between each latitude grid.
         """
-        number = (180.0 / degrees) + 1
-        self.yaxis.set_major_locator(
-            FixedLocator(
-                np.linspace(-np.pi / 2.0, np.pi / 2.0, number, True)[1:-1]))
-        self._latitude_degrees = degrees
+        # Skip -90 and 90, which are the fixed limits.
+        grid = np.arange(-90 + degrees, 90, degrees)
+        self.yaxis.set_major_locator(FixedLocator(np.deg2rad(grid)))
         self.yaxis.set_major_formatter(self.ThetaFormatter(degrees))
 
     def set_longitude_grid_ends(self, degrees):
         """
         Set the latitude(s) at which to stop drawing the longitude grids.
         """
-        self._longitude_cap = degrees * (np.pi / 180.0)
+        self._longitude_cap = np.deg2rad(degrees)
         self._xaxis_pretransform \
             .clear() \
             .scale(1.0, self._longitude_cap * 2.0) \
@@ -252,65 +238,64 @@ class GeoAxes(Axes):
         pass
 
 
+class _GeoTransform(Transform):
+    # Factoring out some common functionality.
+    input_dims = 2
+    output_dims = 2
+    is_separable = False
+
+    def __init__(self, resolution):
+        """
+        Create a new geographical transform.
+
+        Resolution is the number of steps to interpolate between each input
+        line segment to approximate its path in curved space.
+        """
+        Transform.__init__(self)
+        self._resolution = resolution
+
+    def __str__(self):
+        return "{}({})".format(type(self).__name__, self._resolution)
+
+    def transform_path_non_affine(self, path):
+        vertices = path.vertices
+        ipath = path.interpolated(self._resolution)
+        return Path(self.transform(ipath.vertices), ipath.codes)
+    transform_path_non_affine.__doc__ = \
+        Transform.transform_path_non_affine.__doc__
+
+
 class AitoffAxes(GeoAxes):
     name = 'aitoff'
 
-    class AitoffTransform(Transform):
-        """
-        The base Aitoff transform.
-        """
-        input_dims = 2
-        output_dims = 2
-        is_separable = False
-
-        def __init__(self, resolution):
-            """
-            Create a new Aitoff transform.  Resolution is the number of steps
-            to interpolate between each input line segment to approximate its
-            path in curved Aitoff space.
-            """
-            Transform.__init__(self)
-            self._resolution = resolution
+    class AitoffTransform(_GeoTransform):
+        """The base Aitoff transform."""
 
         def transform_non_affine(self, ll):
-            longitude = ll[:, 0:1]
-            latitude  = ll[:, 1:2]
+            longitude = ll[:, 0]
+            latitude = ll[:, 1]
 
             # Pre-compute some values
             half_long = longitude / 2.0
             cos_latitude = np.cos(latitude)
 
             alpha = np.arccos(cos_latitude * np.cos(half_long))
-            # Mask this array or we'll get divide-by-zero errors
-            alpha = ma.masked_where(alpha == 0.0, alpha)
-            # The numerators also need to be masked so that masked
-            # division will be invoked.
+            # Avoid divide-by-zero errors using same method as NumPy.
+            alpha[alpha == 0.0] = 1e-20
             # We want unnormalized sinc.  numpy.sinc gives us normalized
-            sinc_alpha = ma.sin(alpha) / alpha
+            sinc_alpha = np.sin(alpha) / alpha
 
-            x = (cos_latitude * ma.sin(half_long)) / sinc_alpha
-            y = (ma.sin(latitude) / sinc_alpha)
-            return np.concatenate((x.filled(0), y.filled(0)), 1)
+            xy = np.empty_like(ll, float)
+            xy[:, 0] = (cos_latitude * np.sin(half_long)) / sinc_alpha
+            xy[:, 1] = np.sin(latitude) / sinc_alpha
+            return xy
         transform_non_affine.__doc__ = Transform.transform_non_affine.__doc__
 
-        def transform_path_non_affine(self, path):
-            vertices = path.vertices
-            ipath = path.interpolated(self._resolution)
-            return Path(self.transform(ipath.vertices), ipath.codes)
-        transform_path_non_affine.__doc__ = Transform.transform_path_non_affine.__doc__
-
         def inverted(self):
             return AitoffAxes.InvertedAitoffTransform(self._resolution)
         inverted.__doc__ = Transform.inverted.__doc__
 
-    class InvertedAitoffTransform(Transform):
-        input_dims = 2
-        output_dims = 2
-        is_separable = False
-
-        def __init__(self, resolution):
-            Transform.__init__(self)
-            self._resolution = resolution
+    class InvertedAitoffTransform(_GeoTransform):
 
         def transform_non_affine(self, xy):
             # MGDTODO: Math is hard ;(
@@ -334,22 +319,8 @@ class AitoffAxes(GeoAxes):
 class HammerAxes(GeoAxes):
     name = 'hammer'
 
-    class HammerTransform(Transform):
-        """
-        The base Hammer transform.
-        """
-        input_dims = 2
-        output_dims = 2
-        is_separable = False
-
-        def __init__(self, resolution):
-            """
-            Create a new Hammer transform.  Resolution is the number of steps
-            to interpolate between each input line segment to approximate its
-            path in curved Hammer space.
-            """
-            Transform.__init__(self)
-            self._resolution = resolution
+    class HammerTransform(_GeoTransform):
+        """The base Hammer transform."""
 
         def transform_non_affine(self, ll):
             longitude = ll[:, 0:1]
@@ -366,35 +337,18 @@ class HammerAxes(GeoAxes):
             return np.concatenate((x, y), 1)
         transform_non_affine.__doc__ = Transform.transform_non_affine.__doc__
 
-        def transform_path_non_affine(self, path):
-            vertices = path.vertices
-            ipath = path.interpolated(self._resolution)
-            return Path(self.transform(ipath.vertices), ipath.codes)
-        transform_path_non_affine.__doc__ = Transform.transform_path_non_affine.__doc__
-
         def inverted(self):
             return HammerAxes.InvertedHammerTransform(self._resolution)
         inverted.__doc__ = Transform.inverted.__doc__
 
-    class InvertedHammerTransform(Transform):
-        input_dims = 2
-        output_dims = 2
-        is_separable = False
-
-        def __init__(self, resolution):
-            Transform.__init__(self)
-            self._resolution = resolution
+    class InvertedHammerTransform(_GeoTransform):
 
         def transform_non_affine(self, xy):
-            x = xy[:, 0:1]
-            y = xy[:, 1:2]
-
-            quarter_x = 0.25 * x
-            half_y = 0.5 * y
-            z = np.sqrt(1.0 - quarter_x*quarter_x - half_y*half_y)
-            longitude = 2 * np.arctan((z*x) / (2.0 * (2.0*z*z - 1.0)))
+            x, y = xy.T
+            z = np.sqrt(1 - (x / 4) ** 2 - (y / 2) ** 2)
+            longitude = 2 * np.arctan((z * x) / (2 * (2 * z ** 2 - 1)))
             latitude = np.arcsin(y*z)
-            return np.concatenate((longitude, latitude), 1)
+            return np.column_stack([longitude, latitude])
         transform_non_affine.__doc__ = Transform.transform_non_affine.__doc__
 
         def inverted(self):
@@ -414,26 +368,13 @@ class HammerAxes(GeoAxes):
 class MollweideAxes(GeoAxes):
     name = 'mollweide'
 
-    class MollweideTransform(Transform):
-        """
-        The base Mollweide transform.
-        """
-        input_dims = 2
-        output_dims = 2
-        is_separable = False
-
-        def __init__(self, resolution):
-            """
-            Create a new Mollweide transform.  Resolution is the number of steps
-            to interpolate between each input line segment to approximate its
-            path in curved Mollweide space.
-            """
-            Transform.__init__(self)
-            self._resolution = resolution
+    class MollweideTransform(_GeoTransform):
+        """The base Mollweide transform."""
 
         def transform_non_affine(self, ll):
             def d(theta):
-                delta = -(theta + np.sin(theta) - pi_sin_l) / (1 + np.cos(theta))
+                delta = (-(theta + np.sin(theta) - pi_sin_l)
+                         / (1 + np.cos(theta)))
                 return delta, np.abs(delta) > 0.001
 
             longitude = ll[:, 0]
@@ -442,7 +383,7 @@ class MollweideAxes(GeoAxes):
             clat = np.pi/2 - np.abs(latitude)
             ihigh = clat < 0.087 # within 5 degrees of the poles
             ilow = ~ihigh
-            aux = np.empty(latitude.shape, dtype=np.float)
+            aux = np.empty(latitude.shape, dtype=float)
 
             if ilow.any():  # Newton-Raphson iteration
                 pi_sin_l = np.pi * np.sin(latitude[ilow])
@@ -458,31 +399,18 @@ class MollweideAxes(GeoAxes):
                 d = 0.5 * (3 * np.pi * e**2) ** (1.0/3)
                 aux[ihigh] = (np.pi/2 - d) * np.sign(latitude[ihigh])
 
-            xy = np.empty(ll.shape, dtype=np.float)
+            xy = np.empty(ll.shape, dtype=float)
             xy[:,0] = (2.0 * np.sqrt(2.0) / np.pi) * longitude * np.cos(aux)
             xy[:,1] = np.sqrt(2.0) * np.sin(aux)
 
             return xy
         transform_non_affine.__doc__ = Transform.transform_non_affine.__doc__
 
-        def transform_path_non_affine(self, path):
-            vertices = path.vertices
-            ipath = path.interpolated(self._resolution)
-            return Path(self.transform(ipath.vertices), ipath.codes)
-        transform_path_non_affine.__doc__ = Transform.transform_path_non_affine.__doc__
-
         def inverted(self):
             return MollweideAxes.InvertedMollweideTransform(self._resolution)
         inverted.__doc__ = Transform.inverted.__doc__
 
-    class InvertedMollweideTransform(Transform):
-        input_dims = 2
-        output_dims = 2
-        is_separable = False
-
-        def __init__(self, resolution):
-            Transform.__init__(self)
-            self._resolution = resolution
+    class InvertedMollweideTransform(_GeoTransform):
 
         def transform_non_affine(self, xy):
             x = xy[:, 0:1]
@@ -514,13 +442,8 @@ class MollweideAxes(GeoAxes):
 class LambertAxes(GeoAxes):
     name = 'lambert'
 
-    class LambertTransform(Transform):
-        """
-        The base Lambert transform.
-        """
-        input_dims = 2
-        output_dims = 2
-        is_separable = False
+    class LambertTransform(_GeoTransform):
+        """The base Lambert transform."""
 
         def __init__(self, center_longitude, center_latitude, resolution):
             """
@@ -528,8 +451,7 @@ class LambertAxes(GeoAxes):
             to interpolate between each input line segment to approximate its
             path in curved Lambert space.
             """
-            Transform.__init__(self)
-            self._resolution = resolution
+            _GeoTransform.__init__(self, resolution)
             self._center_longitude = center_longitude
             self._center_latitude = center_latitude
 
@@ -556,12 +478,6 @@ class LambertAxes(GeoAxes):
             return np.concatenate((x, y), 1)
         transform_non_affine.__doc__ = Transform.transform_non_affine.__doc__
 
-        def transform_path_non_affine(self, path):
-            vertices = path.vertices
-            ipath = path.interpolated(self._resolution)
-            return Path(self.transform(ipath.vertices), ipath.codes)
-        transform_path_non_affine.__doc__ = Transform.transform_path_non_affine.__doc__
-
         def inverted(self):
             return LambertAxes.InvertedLambertTransform(
                 self._center_longitude,
@@ -569,14 +485,10 @@ class LambertAxes(GeoAxes):
                 self._resolution)
         inverted.__doc__ = Transform.inverted.__doc__
 
-    class InvertedLambertTransform(Transform):
-        input_dims = 2
-        output_dims = 2
-        is_separable = False
+    class InvertedLambertTransform(_GeoTransform):
 
         def __init__(self, center_longitude, center_latitude, resolution):
-            Transform.__init__(self)
-            self._resolution = resolution
+            _GeoTransform.__init__(self, resolution)
             self._center_longitude = center_longitude
             self._center_latitude = center_latitude
 
@@ -606,10 +518,10 @@ class LambertAxes(GeoAxes):
                 self._resolution)
         inverted.__doc__ = Transform.inverted.__doc__
 
-    def __init__(self, *args, **kwargs):
-        self._longitude_cap = np.pi / 2.0
-        self._center_longitude = kwargs.pop("center_longitude", 0.0)
-        self._center_latitude = kwargs.pop("center_latitude", 0.0)
+    def __init__(self, *args, center_longitude=0, center_latitude=0, **kwargs):
+        self._longitude_cap = np.pi / 2
+        self._center_longitude = center_longitude
+        self._center_latitude = center_latitude
         GeoAxes.__init__(self, *args, **kwargs)
         self.set_aspect('equal', adjustable='box', anchor='C')
         self.cla()
